import { AngularFireAuth } from '@angular/fire/auth';
import { AngularFirestore } from '@angular/fire/firestore';
import { auth, User, firestore } from 'firebase/app';
import 'firebase/auth';
import { Observable } from 'rxjs';
import { Store, UpdateStateCallback } from '@datorama/akita';
import { FireAuthState } from './auth.model';
import { WriteOptions } from '../utils/types';
export declare const fireAuthProviders: readonly ["github", "google", "microsoft", "facebook", "twitter", "email"];
declare type FireProvider = (typeof fireAuthProviders)[number];
/** Verify if provider is part of the list of Authentication provider provided by Firebase Auth */
export declare function isFireAuthProvider(provider: string): provider is FireProvider;
/**
 * Get the custom claims of a user. If no key is provided, return the whole claims object
 * @param user The user object returned by Firebase Auth
 * @param roles Keys of the custom claims inside the claim objet
 */
export declare function getCustomClaims(user: User, roles?: string | string[]): Promise<Record<string, any>>;
/**
 * Get the Authentication Provider based on its name
 * @param provider string literal representing the name of the provider
 */
export declare function getAuthProvider(provider: FireProvider): auth.EmailAuthProvider | auth.FacebookAuthProvider | auth.GithubAuthProvider | auth.GoogleAuthProvider | auth.OAuthProvider | auth.TwitterAuthProvider;
export declare class FireAuthService<S extends FireAuthState> {
    protected store: Store<S>;
    private collection;
    protected collectionPath: string;
    protected fireAuth: AngularFireAuth;
    protected db: AngularFirestore;
    protected onCreate?(profile: S['profile'], write: WriteOptions): any;
    protected onUpdate?(profile: S['profile'], write: WriteOptions): any;
    protected onDelete?(write: WriteOptions): any;
    /** Triggered when user signin for the first time */
    protected onSignup?(user: auth.UserCredential): any;
    /** Triggered when a user signin, except for the first time @see onSignup */
    protected onSignin?(user: auth.UserCredential): any;
    constructor(store: Store<S>);
    readonly idKey: any;
    /** Can be overrided */
    protected selectProfile(user: User): Observable<S['profile']>;
    /** Can be overrided */
    protected selectRoles(user: User): Promise<S['roles']> | Observable<S['roles']>;
    /**
     * Function triggered when getting data from firestore
     * @note should be overrided
     */
    protected formatFromFirestore<DB>(user: DB): S['profile'];
    /**
     * Function triggered when adding/updating data to firestore
     * @note should be overrided
     */
    protected formatToFirestore<DB>(user: S['profile']): DB;
    /**
     * Function triggered when transforming a user into a profile
     * @note Should be override
     */
    protected createProfile(user: User): Promise<Partial<S['profile']>> | Partial<S['profile']>;
    /** The current sign-in user (or null) */
    readonly user: User;
    /** The path to the profile in firestore */
    readonly path: any;
    /** Start listening on User */
    sync(): Observable<any[]>;
    /**
     * @description Delete user from authentication service and database
     * WARNING This is security sensitive operation
     */
    delete(options?: WriteOptions): Promise<void>;
    /** Update the current profile of the authenticated user */
    update(profile: Partial<S['profile']> | UpdateStateCallback<S['profile']>, options?: WriteOptions): Promise<void | firestore.Transaction>;
    /** Create a user based on email and password */
    signup(email: string, password: string): Promise<auth.UserCredential>;
    signin(email: string, password: string): Promise<auth.UserCredential>;
    signin(provider?: FireProvider): Promise<auth.UserCredential>;
    signin(token: string): Promise<auth.UserCredential>;
    /** Signs out the current user */
    signOut(): Promise<void>;
}
export {};
